@@ -247,7 +247,7 @@ pub fn emit_variable_length<E: UserDefinedError>(jit: &mut JitCompiler, size: Op
     }
 }
 
-#[derive(PartialEq, Eq, Copy, Clone)]
+#[derive(PartialEq, Eq, Copy, Clone, Debug)]
 pub enum OperandSize {
     S0  = 0,
     S8  = 8,
@@ -586,7 +586,7 @@ fn emit_bpf_call<E: UserDefinedError>(jit: &mut JitCompiler, dst: Value) -> Resu
             emit_validate_and_profile_instruction_count(jit, false, None)?;
             X86Instruction::mov(OperandSize::S64, REGISTER_MAP[0], R11).emit(jit)?; // Save target_pc
             X86Instruction::pop(REGISTER_MAP[0]).emit(jit)?; // Restore RAX
-            X86Instruction::call_reg(OperandSize::S64, R11, None).emit(jit)?; // callq *%r11
+            X86Instruction::call_reg(R11, None).emit(jit)?; // callq *%r11
         },
         Value::Constant64(target_pc, user_provided) => {
             debug_assert!(!user_provided);
@@ -701,12 +701,12 @@ fn emit_rust_call<E: UserDefinedError>(jit: &mut JitCompiler, dst: Value, argume
 
     match dst {
         Value::Register(reg) => {
-            X86Instruction::call_reg(OperandSize::S64, reg, None).emit(jit)?;
+            X86Instruction::call_reg(reg, None).emit(jit)?;
         },
         Value::Constant64(value, user_provided) => {
             debug_assert!(!user_provided);
             X86Instruction::load_immediate(OperandSize::S64, RAX, value).emit(jit)?;
-            X86Instruction::call_reg(OperandSize::S64, RAX, None).emit(jit)?;
+            X86Instruction::call_reg(RAX, None).emit(jit)?;
         },
         _ => {
             #[cfg(debug_assertions)]
@@ -1487,13 +1487,13 @@ impl JitCompiler {
             emit_alu(self, OperandSize::S64, 0xc1, 4, RAX, 5, None)?; // RAX *= mem::size_of::<MemoryRegion>();
             emit_alu(self, OperandSize::S64, 0x03, RAX, R10, 0, Some(X86IndirectAccess::Offset(self.program_argument_key)))?; // region = &jit_program_argument.memory_mapping.regions[region_index];
             if *access_type == AccessType::Store {
-                X86Instruction::cmp_immediate(OperandSize::S8, RAX, 0, Some(X86IndirectAccess::Offset(25))).emit(self)?; // region.is_writable == 0
+                X86Instruction::cmp_immediate(OperandSize::S8, RAX, 0, Some(X86IndirectAccess::Offset(MemoryRegion::IS_WRITABLE_OFFSET))).emit(self)?; // region.is_writable == 0
                 emit_jcc(self, 0x84, TARGET_PC_MEMORY_ACCESS_VIOLATION + target_offset)?;
             }
             X86Instruction::load_immediate(OperandSize::S64, RCX, (1i64 << ebpf::VIRTUAL_ADDRESS_BITS) - 1).emit(self)?; // RCX = (1 << ebpf::VIRTUAL_ADDRESS_BITS) - 1;
             emit_alu(self, OperandSize::S64, 0x21, RCX, R11, 0, None)?; // R11 &= (1 << ebpf::VIRTUAL_ADDRESS_BITS) - 1;
             if self.config.enable_stack_frame_gaps {
-                X86Instruction::load(OperandSize::S8, RAX, RCX, X86IndirectAccess::Offset(24)).emit(self)?; // RCX = region.vm_gap_shift;
+                X86Instruction::load(OperandSize::S8, RAX, RCX, X86IndirectAccess::Offset(MemoryRegion::VM_GAP_SHIFT_OFFSET)).emit(self)?; // RCX = region.vm_gap_shift;
                 X86Instruction::mov(OperandSize::S64, R11, RDX).emit(self)?; // RDX = R11;
                 emit_alu(self, OperandSize::S64, 0xd3, 5, RDX, 0, None)?; // RDX = R11 >> region.vm_gap_shift;
                 X86Instruction::test_immediate(OperandSize::S64, RDX, 1, None).emit(self)?; // (RDX & 1) != 0
@@ -1508,9 +1508,9 @@ impl JitCompiler {
                 emit_alu(self, OperandSize::S64, 0x09, RCX, R11, 0, None)?; // gapped_offset = above_gap | below_gap;
             }
             X86Instruction::lea(OperandSize::S64, R11, RCX, Some(X86IndirectAccess::Offset(*len))).emit(self)?; // RCX = R11 + len;
-            X86Instruction::cmp(OperandSize::S64, RCX, RAX, Some(X86IndirectAccess::Offset(16))).emit(self)?; // region.len < R11 + len
+            X86Instruction::cmp(OperandSize::S64, RCX, RAX, Some(X86IndirectAccess::Offset(MemoryRegion::LEN_OFFSET))).emit(self)?; // region.len < R11 + len
             emit_jcc(self, 0x82, TARGET_PC_MEMORY_ACCESS_VIOLATION + target_offset)?;
-            emit_alu(self, OperandSize::S64, 0x03, R11, RAX, 0, Some(X86IndirectAccess::Offset(0)))?; // R11 += region.host_addr;
+            emit_alu(self, OperandSize::S64, 0x03, R11, RAX, 0, Some(X86IndirectAccess::Offset(MemoryRegion::HOST_ADDR_OFFSET)))?; // R11 += region.host_addr;
             if self.config.enable_stack_frame_gaps {
                 X86Instruction::pop(RDX).emit(self)?;
             }
